'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = operator;

var _mitt = require('mitt');

var _mitt2 = _interopRequireDefault(_mitt);

var _unfetch = require('unfetch');

var _unfetch2 = _interopRequireDefault(_unfetch);

var _scrollRestoration = require('scroll-restoration');

var _scrollRestoration2 = _interopRequireDefault(_scrollRestoration);

var _cache = require('./lib/cache.js');

var _cache2 = _interopRequireDefault(_cache);

var _util = require('./lib/util.js');

var _routes = require('./lib/routes.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function operator(_ref) {
  var _ref$root = _ref.root,
      root = _ref$root === undefined ? 'root' : _ref$root,
      _ref$transitionSpeed = _ref.transitionSpeed,
      transitionSpeed = _ref$transitionSpeed === undefined ? 0 : _ref$transitionSpeed,
      _ref$routes = _ref.routes,
      routes = _ref$routes === undefined ? {} : _ref$routes,
      _ref$evaluateScripts = _ref.evaluateScripts,
      evaluateScripts = _ref$evaluateScripts === undefined ? false : _ref$evaluateScripts;

  if (!window.history.pushState) {
    return console.error('operator: the history API is unavailable, aborting.');
  }

  /**
   * Take control of scroll position
   */
  _scrollRestoration2.default.init();

  /**
   * Changed via enable()/disable() methods
   */
  var ajaxDisabled = false;

  /**
   * Emitter instance
   */
  var ev = (0, _mitt2.default)();

  /**
   * Map over routes to create pattern
   * matching handlers
   */
  routes = Object.keys(routes).map(function (k) {
    return (0, _routes.createRoute)(k, routes[k]);
  });

  /**
   * Update active links to match initial
   * page load
   */
  (0, _util.setActiveLinks)(_util.location.href);

  /**
   * @param {string} markup The new markup from a successful request
   * @param {string} href The new URL
   * @param {boolean} isPopstate True if render is called via popstate, false otherwise
   */
  function render(markup, href, isPopstate) {
    var mountNode = document.getElementById(root);
    var oldDom = document;
    var newDom = new window.DOMParser().parseFromString(markup, 'text/html');
    var title = newDom.title;

    ev.emit('beforeRender', href);

    document.documentElement.classList.add('operator-is-transitioning');
    mountNode.style.height = mountNode.clientHeight + 'px';

    /**
     * After transition out, render new page
     * and (optionally) push a new history location
     */
    setTimeout(function () {
      mountNode.innerHTML = newDom.getElementById(root).innerHTML;

      /**
       * If a popstate event occurred, we don't
       * need to manually create a new history
       * location: it's already there from
       * a previous navigation
       */
      !isPopstate && instance.push(href, title);

      /**
       * Finish up
       */
      setTimeout(function () {
        mountNode.style.height = '';
        document.documentElement.classList.remove('operator-is-transitioning');
        (0, _util.setActiveLinks)(href);
        ev.emit('afterRender', href);
        evaluateScripts && (0, _util.evalScripts)(newDom, oldDom);
        _scrollRestoration2.default.restore();
      }, 0);
    }, transitionSpeed);
  }

  function handleClick(e) {
    if (ajaxDisabled) return;

    var target = e.target;

    /**
     * Find link that was clicked
     */
    while (target && !(target.href && target.nodeName === 'A')) {
      target = target.parentNode;
    }

    /**
     * Validate URL
     */
    var href = (0, _util.getValidPath)(e, target);

    if (href) {
      e.preventDefault();

      if ((0, _util.isSameURL)(href)) {
        if ((0, _util.isHash)(href)) {
          instance.push(href);
          ev.emit('afterRender', href);
        }

        return;
      }

      /**
       * Only save on clicks, not on popstate
       */
      _scrollRestoration2.default.save();

      instance.go(href);

      return false;
    }
  }

  /**
   * Notes on popstate:
   *  - catches hashchange
   *  - must validate url before AJAX
   */
  function onPopstate(e) {
    if (ajaxDisabled) return;

    e.preventDefault();

    /**
     * If it's a back button, the
     * target should be a window object.
     * Otherwise this could be a hash
     * link or otherwise.
     */
    var path = e.target.window ? e.target.window.location.href : (0, _util.getValidPath)(e, e.target);

    if (path) {
      instance.go(e.target.location.href, true); // set isPopstate to true

      return false;
    }
  }

  var instance = _extends({}, ev, {
    go: function go(href, isPopstate) {
      var _this = this;

      href = (0, _util.getAnchor)(href).href; // ensure it's a full address

      (0, _routes.executeRoute)(href, routes, function (redirect) {
        if (redirect) return _this.go(redirect);

        _this.prefetch(href).then(function (markup) {
          return render(markup, href, isPopstate);
        });
      });
    },
    push: function push(route) {
      var title = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : document.title;

      window.history.pushState({}, title, route);
      document.title = title;
    },
    prefetch: function prefetch(route) {
      var cached = _cache2.default.get(route);
      return cached ? Promise.resolve(cached) : (0, _unfetch2.default)(route, { credentials: 'include' }).then(function (res) {
        return res.text();
      }).then(function (markup) {
        _cache2.default.set(route, markup);
        return markup;
      });
    },
    addRoute: function addRoute(route, handler) {
      routes.push((0, _routes.createRoute)(route, handler));
    },
    disable: function disable() {
      ajaxDisabled = true;
    },
    enable: function enable() {
      ajaxDisabled = false;
    },
    isEnabled: function isEnabled() {
      return ajaxDisabled;
    },
    destroy: function destroy() {
      document.body.removeEventListener('click', handleClick);
      window.removeEventListener('popstate', onPopstate);
    }
  });

  document.body.addEventListener('click', handleClick);
  window.addEventListener('popstate', onPopstate);

  /**
   * Runs any applicable routes on page load,
   * restore scroll (if saved at history.state.scrollPosition)
   * *after* routes are fired
   */
  (0, _routes.executeRoute)(window.location.pathname, routes, function (redirect) {
    if (redirect) return instance.go(redirect);
    _scrollRestoration2.default.restore();
  });

  return instance;
}