'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.collectParams = collectParams;
exports.createRoute = createRoute;
exports.executeRoute = executeRoute;
/**
 * route utils lifted and adapted from
 * dush-router by @@tunnckoCore
 * @see https://github.com/tunnckoCore/dush-router
 */
function collectParams(r, pathname) {
  var match = null;

  pathname.replace(r.regex, function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var _loop = function _loop(i) {
      r.keys.forEach(function (key) {
        r.params[key] = args[i];
      });
      match = true;
    };

    for (var i = 1; i < args.length - 2; i++) {
      _loop(i);
    }
  });

  return match ? r.params : match;
}

function createRoute(route, handler) {
  var keys = [];

  var regex = new RegExp(route.replace(/\*/g, '(?:.*)').replace(/([:*])(\w+)/g, function (key) {
    keys.push(key.slice(1));
    return '([\\w-]+)';
  }) + '(?:[\/|?\w+]$|$)' + '$', 'ig');

  return {
    route: route,
    handler: handler,
    regex: regex,
    keys: keys,
    params: {},
    match: function match(pathname) {
      return regex.test(pathname) ? collectParams(this, pathname) : false;
    }
  };
}

function executeRoute(pathname, routes, done) {
  if (routes.length < 1) return done && done();

  var handlers = [];

  /**
   * If we have configured routes,
   * check them and fire any handlers
   */
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = routes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var route = _step.value;

      var params = route.match(pathname);
      /**
       * params will return be `null` if
       * there was a match, but not parametized
       * route params. If params is `false`,
       * it means a no-match, so skip the handler
       */
      if (params === false) {
        continue;
      }

      handlers.push(route.handler(params || {}, pathname));
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  Promise.all(handlers).then(function (responses) {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = responses[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var response = _step2.value;

        if (typeof response === 'string') return done(response); // handle redirect
        if (response === false && window.location.pathname !== pathname) {
          return window.location = pathname;
        }
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    done && done();
  });
}