'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getOrigin = getOrigin;
exports.getAnchor = getAnchor;
exports.sanitize = sanitize;
exports.isHash = isHash;
exports.isSameURL = isSameURL;
exports.isSameOrigin = isSameOrigin;
exports.getValidPath = getValidPath;
exports.setActiveLinks = setActiveLinks;
exports.evalScripts = evalScripts;
var location = exports.location = window.location;

function getOrigin(loc) {
  var _ref = loc || window.location,
      protocol = _ref.protocol,
      host = _ref.host;

  return protocol + '//' + host;
}

function getAnchor(url) {
  var a = document.createElement('a');
  a.href = url;
  return a;
}

/**
 * @param {string} url Raw URL to parse
 * @return {string} URL sans origin
 */
function sanitize(url) {
  var route = url.replace(new RegExp(getOrigin()), '');
  return route;
}

function isHash(href) {
  return (/#/.test(href)
  );
}

function isSameURL(href) {
  return window.location.search === getAnchor(href).search && window.location.pathname === getAnchor(href).pathname;
}

function isSameOrigin(href) {
  return getOrigin() === getOrigin(getAnchor(href));
}

function getValidPath(e, target) {
  if (e.metaKey || e.ctrlKey || e.shiftKey || e.altKey) return;
  if (!target) return;
  if (!target.href) return;
  if (target.target === '_blank') return;
  if (!isSameOrigin(target.href)) return;
  if (target.classList.contains('no-ajax')) return;
  return target.href;
}

var activeLinks = [];

function setActiveLinks(href) {
  var route = getAnchor(href).pathname;
  var regex = /^\/$/.test(route) ? RegExp(/^\/$/) : new RegExp(route);

  for (var i = 0; i < activeLinks.length; i++) {
    activeLinks[i].classList.remove('is-active');
  }

  activeLinks = [].slice.call(document.querySelectorAll('[href$="' + route + '"]'));

  for (var _i = 0; _i < activeLinks.length; _i++) {
    if (regex.test(sanitize(activeLinks[_i].href))) {
      activeLinks[_i].classList.add('is-active');
    }
  }
}

function evalScripts(newDom, existingDom) {
  var existing = Array.prototype.slice.call(existingDom.getElementsByTagName('script'));
  var scripts = newDom.getElementsByTagName('script');

  var _loop = function _loop(i) {
    if (existing.filter(function (e) {
      return e.isEqualNode(scripts[i]);
    }).length > 0) {
      return 'continue';
    }

    var s = document.createElement('script');

    for (var a = 0; a < scripts[i].attributes.length; a++) {
      var attr = scripts[i].attributes[a];
      s.setAttribute(attr.name, attr.value);
    }

    if (!s.src) {
      s.innerHTML = scripts[i].innerHTML;
    }

    document.body.appendChild(s);
  };

  for (var i = 0; i < scripts.length; i++) {
    var _ret = _loop(i);

    if (_ret === 'continue') continue;
  }
}